@@ -73,7 +73,6 @@ if six.PY2:
 DEBUG = False
 WAIT_FOR_DEVICE_TIMEOUT = int(os.getenv("WAIT_FOR_DEVICE_TIMEOUT", 20))
 
-
 log_format = '%(color)s[%(levelname)1.1s %(asctime)s %(module)s:%(lineno)d]%(end_color)s [pid:%(process)d] %(message)s'
 formatter = logzero.LogFormatter(fmt=log_format)
 logger = setup_logger("uiautomator2", level=logging.DEBUG, formatter=formatter)
@@ -82,6 +81,7 @@ _mswindows = (os.name == "nt")
 
 class AppInstaller(abc.ABC):
     """ 解决手机安装apk弹窗问题 """
+
     @abc.abstractmethod
     def install(self, url: str):
         """ install app to device """
@@ -210,7 +210,6 @@ class _AgentRequestSession(TimeoutRequestsSession):
                     self.__client._is_agent_alive():
                 raise
 
-
         if not self.__client._serial:
             raise OSError(
                 "http-request to atx-agent error, can only recover from USB")
@@ -248,7 +247,7 @@ class _BaseClient(object):
             self._serial = None
             self._atx_agent_url = serial_or_url
             return
-        
+
         # USB 连接
         self._serial = serial_or_url
         self._atx_agent_url = None
@@ -262,7 +261,7 @@ class _BaseClient(object):
         wlan_ip = self.wlan_ip
         if wlan_ip:
             self._atx_agent_url = f"http://{wlan_ip}:7912"
-        
+
         filelock_path = os.path.expanduser("~/.uiautomator2/filelocks/") + self._serial.replace(":", "_") + ".lock"
         os.makedirs(os.path.dirname(filelock_path), exist_ok=True)
         self._filelock = filelock.FileLock(filelock_path, timeout=200)
@@ -372,7 +371,7 @@ class _BaseClient(object):
                 adb.wait_for(self._serial, timeout=1)
             except adbutils.AdbTimeout:
                 continue
-            
+
             return adb.device(self._serial)
         return None
 
@@ -385,7 +384,7 @@ class _BaseClient(object):
         self.shell(["pm", "uninstall", "com.github.uiautomator.test"])
 
         assets_dir = Path(__file__).absolute().parent.joinpath("assets")
-        
+
         for name in ("app-release.apk", "app-debug-androidTest.apk"):
             apk_path = assets_dir.joinpath(name).as_posix()
             target_path = "/data/local/tmp/" + name
@@ -430,7 +429,7 @@ class _BaseClient(object):
                                  stream=True)
             # return self._request("get", "/shell/stream", params={"command": cmdline}, timeout=None, stream=True) # yapf: disable
         data = dict(command=cmdline, timeout=str(timeout))
-        ret = self.http.post("/shell", data=data, timeout=timeout+10)
+        ret = self.http.post("/shell", data=data, timeout=timeout + 10)
         if ret.status_code != 200:
             raise RuntimeError(
                 "device agent responds with an error code %d" %
@@ -519,7 +518,7 @@ class _BaseClient(object):
         if res.status_code == 502:
             raise GatewayError(
                 res, "gateway error, time used %.1fs" %
-                (time.time() - request_start))
+                     (time.time() - request_start))
         if res.status_code == 410:  # http status gone: session broken
             raise SessionBrokenError("app quit or crash", res.text)
         if res.status_code != 200:
@@ -576,7 +575,7 @@ class _BaseClient(object):
             return r.text.strip()
         except requests.RequestException as e:
             return None
-    
+
     def _is_agent_outdated(self) -> bool:
         version = self._get_agent_version()
         if version != __atx_agent_version__:
@@ -635,7 +634,7 @@ class _BaseClient(object):
 
             # atx-agent might be outdated, check atx-agent version here
             if self._is_agent_outdated():
-                if self._serial: # update atx-agent will not work on WiFi
+                if self._serial:  # update atx-agent will not work on WiFi
                     self._prepare_atx_agent()
 
             ok = self._force_reset_uiautomator_v2(
@@ -650,7 +649,7 @@ class _BaseClient(object):
                 reason = "signature not match, reinstall uiautomator apks"
 
         return self.reset_uiautomator(reason=reason,
-                                    depth=depth + 1)
+                                      depth=depth + 1)
 
     def _force_reset_uiautomator_v2(self, launch_test_app=False):
         brand = self.shell("getprop ro.product.brand").output.strip()
@@ -672,7 +671,7 @@ class _BaseClient(object):
             self._grant_app_permissions()
             self.shell(['am', 'start', '-a', 'android.intent.action.MAIN', '-c',
                         'android.intent.category.LAUNCHER', '-n', package_name + "/" + ".ToastActivity"])
-            
+
         self.uiautomator.start()
 
         # wait until uiautomator2 service is working
@@ -681,7 +680,7 @@ class _BaseClient(object):
         flow_window_showed = False
         while time.time() < deadline:
             self.logger.debug("uiautomator-v2 is starting ... left: %.1fs",
-                         deadline - time.time())
+                              deadline - time.time())
 
             if not self.uiautomator.running():
                 break
@@ -738,9 +737,9 @@ class _BaseClient(object):
     def _grant_app_permissions(self):
         self.logger.debug("grant permissions")
         for permission in [
-                "android.permission.SYSTEM_ALERT_WINDOW",
-                "android.permission.ACCESS_FINE_LOCATION",
-                "android.permission.READ_PHONE_STATE",
+            "android.permission.SYSTEM_ALERT_WINDOW",
+            "android.permission.ACCESS_FINE_LOCATION",
+            "android.permission.READ_PHONE_STATE",
         ]:
             self.shell(['pm', 'grant', "com.github.uiautomator", permission])
 
@@ -1325,10 +1324,8 @@ class _AppMixIn:
         """
         Returns:
             dict(package, activity, pid?)
-
         Raises:
             OSError
-
         For developer:
             Function reset_uiautomator need this function, so can't use jsonrpc here.
         """
@@ -1341,13 +1338,23 @@ class _AppMixIn:
         #   r'mFocusedApp=.*ActivityRecord{\w+ \w+ (?P<package>.*)/(?P<activity>.*) .*'
         #   r'mCurrentFocus=Window{\w+ \w+ (?P<package>.*)/(?P<activity>.*)\}')
         _focusedRE = re.compile(
-            r'mCurrentFocus=Window{.*\s+(?P<package>[^\s]+)/(?P<activity>[^\s]+)\}'
+            r'mCurrentFocus=Window{.*?\s+(?P<package>[^\s]+)/(?P<activity>[^\s]+)\}'
         )
         m = _focusedRE.search(self.shell(['dumpsys', 'window', 'windows'])[0])
         if m:
             return dict(package=m.group('package'),
                         activity=m.group('activity'))
 
+        # search mResumedActivity
+        # https://stackoverflow.com/questions/13193592/adb-android-getting-the-name-of-the-current-activity
+        package = None
+        output, _ = self.shell(['dumpsys', 'activity', 'activities'])
+        _recordRE = re.compile(
+            r'mResumedActivity: ActivityRecord\{.*?\s+(?P<package>[^\s]+)/(?P<activity>[^\s]+)\s.*?\}')
+        m = _recordRE.search(output)
+        if m:
+            package = m.group("package")
+
         # try: adb shell dumpsys activity top
         _activityRE = re.compile(
             r'ACTIVITY (?P<package>[^\s]+)/(?P<activity>[^/\s]+) \w+ pid=(?P<pid>\d+)'
@@ -1359,6 +1366,9 @@ class _AppMixIn:
             ret = dict(package=m.group('package'),
                        activity=m.group('activity'),
                        pid=int(m.group('pid')))
+            if ret['package'] == package:
+                return ret
+
         if ret:  # get last result
             return ret
         raise OSError("Couldn't get focused app")
@@ -1374,7 +1384,7 @@ class _AppMixIn:
         target = "/data/local/tmp/_tmp.apk"
         self.push(data, target, show_progress=True)
         logger.debug("pm install -r -t %s", target)
-        ret = self.shell(['pm', 'install', "-r", "-t", target],timeout=300)
+        ret = self.shell(['pm', 'install', "-r", "-t", target], timeout=300)
         if ret.exit_code != 0:
             raise RuntimeError(ret.output, ret.exit_code)
 
@@ -1395,7 +1405,8 @@ class _AppMixIn:
             time.sleep(.5)
         return False
 
-    def app_start(self, package_name: str, activity: Optional[str] = None, wait: bool = False, stop: bool = False, use_monkey: bool = False):
+    def app_start(self, package_name: str, activity: Optional[str] = None, wait: bool = False, stop: bool = False,
+                  use_monkey: bool = False):
         """ Launch application
         Args:
             package_name (str): package name
